@@ -216,22 +216,9 @@ static void raio_finalizer(SEXP xptr) {
 
 }
 
-// core aio --------------------------------------------------------------------
+// core aio - internal ---------------------------------------------------------
 
-SEXP rnng_aio_result(SEXP env) {
-
-  const SEXP exist = Rf_findVarInFrame(env, nano_ValueSymbol);
-  if (exist != R_UnboundValue)
-    return exist;
-
-  const SEXP aio = Rf_findVarInFrame(env, nano_AioSymbol);
-  if (NANO_PTR_CHECK(aio, nano_AioSymbol))
-    Rf_error("object is not a valid Aio");
-
-  nano_aio *saio = (nano_aio *) NANO_PTR(aio);
-
-  if (nng_aio_busy(saio->aio))
-    return nano_unresolved;
+static inline SEXP create_aio_result(SEXP env, nano_aio *saio) {
 
   if (saio->result > 0)
     return mk_error_aio(saio->result, env);
@@ -242,6 +229,93 @@ SEXP rnng_aio_result(SEXP env) {
 
 }
 
+static inline SEXP create_aio_msg(SEXP env, SEXP aio, nano_aio *raio, int res) {
+
+  SEXP out, pipe;
+  unsigned char *buf;
+  size_t sz;
+
+  if (raio->type == IOV_RECVAIO || raio->type == IOV_RECVAIOS) {
+    buf = raio->data;
+    sz = nng_aio_count(raio->aio);
+  } else {
+    nng_msg *msg = (nng_msg *) raio->data;
+    buf = nng_msg_body(msg);
+    sz = nng_msg_len(msg);
+  }
+
+  PROTECT(out = nano_decode(buf, sz, raio->mode, NANO_PROT(aio)));
+  PROTECT(pipe = Rf_ScalarInteger(-res));
+  Rf_defineVar(nano_ValueSymbol, out, env);
+  Rf_defineVar(nano_AioSymbol, pipe, env);
+
+  UNPROTECT(2);
+  return out;
+
+}
+
+
+SEXP nano_aio_result(SEXP env) {
+
+  const SEXP exist = Rf_findVarInFrame(env, nano_ValueSymbol);
+  if (exist != R_UnboundValue)
+    return exist;
+
+  const SEXP aio = Rf_findVarInFrame(env, nano_AioSymbol);
+  nano_aio *saio = (nano_aio *) NANO_PTR(aio);
+
+  return create_aio_result(env, saio);
+
+}
+
+SEXP nano_aio_get_msg(SEXP env) {
+
+  const SEXP exist = Rf_findVarInFrame(env, nano_ValueSymbol);
+  if (exist != R_UnboundValue)
+    return exist;
+
+  const SEXP aio = Rf_findVarInFrame(env, nano_AioSymbol);
+  nano_aio *raio = (nano_aio *) NANO_PTR(aio);
+
+  int res;
+  switch (raio->type) {
+  case RECVAIO:
+  case REQAIO:
+  case IOV_RECVAIO:
+  case RECVAIOS:
+  case REQAIOS:
+  case IOV_RECVAIOS: ;
+    res = raio->result;
+    if (res > 0)
+      return mk_error_aio(res, env);
+    break;
+  default:
+    /* not reached */
+    res = 0;
+  }
+
+  return create_aio_msg(env, aio, raio, res);
+
+}
+
+// core aio --------------------------------------------------------------------
+
+SEXP rnng_aio_result(SEXP env) {
+
+  const SEXP exist = Rf_findVarInFrame(env, nano_ValueSymbol);
+  if (exist != R_UnboundValue)
+    return exist;
+
+  const SEXP aio = Rf_findVarInFrame(env, nano_AioSymbol);
+  nano_aio *saio = (nano_aio *) NANO_PTR(aio);
+
+  if (nng_aio_busy(saio->aio))
+    return nano_unresolved;
+
+  return create_aio_result(env, saio);
+
+}
+
 SEXP rnng_aio_get_msg(SEXP env) {
 
   const SEXP exist = Rf_findVarInFrame(env, nano_ValueSymbol);
@@ -249,9 +323,6 @@ SEXP rnng_aio_get_msg(SEXP env) {
     return exist;
 
   const SEXP aio = Rf_findVarInFrame(env, nano_AioSymbol);
-  if (NANO_PTR_CHECK(aio, nano_AioSymbol))
-    Rf_error("object is not a valid Aio");
-
   nano_aio *raio = (nano_aio *) NANO_PTR(aio);
 
   int res;
@@ -283,30 +354,11 @@ SEXP rnng_aio_get_msg(SEXP env) {
 
     break;
   default:
+    /* not reached */
     res = 0;
-    return mk_error_aio(res, env);
   }
 
-  SEXP out, pipe;
-  unsigned char *buf;
-  size_t sz;
-
-  if (raio->type == IOV_RECVAIO || raio->type == IOV_RECVAIOS) {
-    buf = raio->data;
-    sz = nng_aio_count(raio->aio);
-  } else {
-    nng_msg *msg = (nng_msg *) raio->data;
-    buf = nng_msg_body(msg);
-    sz = nng_msg_len(msg);
-  }
-
-  PROTECT(out = nano_decode(buf, sz, raio->mode, NANO_PROT(aio)));
-  PROTECT(pipe = Rf_ScalarInteger(-res));
-  Rf_defineVar(nano_ValueSymbol, out, env);
-  Rf_defineVar(nano_AioSymbol, pipe, env);
-
-  UNPROTECT(2);
-  return out;
+  return create_aio_msg(env, aio, raio, res);
 
 }
 
@@ -327,14 +379,14 @@ SEXP rnng_aio_call(SEXP x) {
     case RECVAIOS:
     case REQAIOS:
     case IOV_RECVAIOS:
-      rnng_aio_get_msg(x);
+      nano_aio_get_msg(x);
       break;
     case SENDAIO:
     case IOV_SENDAIO:
-      rnng_aio_result(x);
+      nano_aio_result(x);
       break;
     case HTTP_AIO:
-      rnng_aio_http_status(x);
+      nano_aio_http_status(x);
       break;
     }
     break;
