@@ -208,71 +208,6 @@ void tls_finalizer(SEXP xptr) {
 
 }
 
-#if R_VERSION < R_Version(4, 1, 0)
-
-inline SEXP R_NewEnv(SEXP parent, int hash, int size) {
-  (void) parent;
-  (void) hash;
-  (void) size;
-  return Rf_allocSExp(ENVSXP);
-}
-
-#endif
-
-#if R_VERSION < R_Version(4, 5, 0)
-
-inline SEXP R_mkClosure(SEXP formals, SEXP body, SEXP env) {
-  SEXP fun = Rf_allocSExp(CLOSXP);
-  SET_FORMALS(fun, formals);
-  SET_BODY(fun, body);
-  SET_CLOENV(fun, env);
-  return fun;
-}
-
-#endif
-
-inline SEXP nano_PreserveObject(const SEXP x) {
-
-  SEXP tail = CDR(nano_precious);
-  SEXP node = Rf_cons(nano_precious, tail);
-  SETCDR(nano_precious, node);
-  if (tail != R_NilValue)
-    SETCAR(tail, node);
-  SET_TAG(node, x);
-
-  return node;
-
-}
-
-inline void nano_ReleaseObject(SEXP node) {
-
-  SET_TAG(node, R_NilValue);
-  SEXP head = CAR(node);
-  SEXP tail = CDR(node);
-  SETCDR(head, tail);
-  if (tail != R_NilValue)
-    SETCAR(tail, head);
-
-}
-
-void later2(void (*fun)(void *), void *data) {
-  eln2(fun, data, 0, 0);
-}
-
-void raio_invoke_cb(void *arg) {
-
-  SEXP call, node = (SEXP) arg, x = TAG(node);
-  PROTECT(call = Rf_lcons(nano_ResolveSymbol, Rf_cons(rnng_aio_get_msg(x), R_NilValue)));
-  Rf_eval(call, NANO_ENCLOS(x));
-  UNPROTECT(1);
-  nano_ReleaseObject(node);
-
-}
-
-inline int nano_integer(const SEXP x) {
-  return (TYPEOF(x) == INTSXP || TYPEOF(x) == LGLSXP) ? NANO_INTEGER(x) : Rf_asInteger(x);
-}
-
 SEXP mk_error(const int xc) {
 
   SEXP err = Rf_ScalarInteger(xc);
